@@ -1,9 +1,9 @@
 package App::FirefoxUtils;
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2020-05-24'; # DATE
+our $DATE = '2020-06-13'; # DATE
 our $DIST = 'App-FirefoxUtils'; # DIST
-our $VERSION = '0.012'; # VERSION
+our $VERSION = '0.013'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
@@ -53,6 +53,18 @@ sub unpause_firefox {
     App::BrowserUtils::_do_browser('unpause', 'firefox', @_);
 }
 
+$SPEC{firefox_has_processes} = {
+    v => 1.1,
+    summary => "Check whether Firefox has processes",
+    args => {
+        %App::BrowserUtils::args_common,
+        %App::BrowserUtils::argopt_quiet,
+    },
+};
+sub firefox_has_processes {
+    App::BrowserUtils::_do_browser('has_processes', 'firefox', @_);
+}
+
 $SPEC{firefox_is_paused} = {
     v => 1.1,
     summary => "Check whether Firefox is paused",
@@ -101,6 +113,36 @@ sub terminate_firefox {
     App::BrowserUtils::_do_browser('terminate', 'firefox', @_);
 }
 
+$SPEC{restart_firefox} = {
+    v => 1.1,
+    summary => "Restart firefox",
+    args => {
+        %App::BrowserUtils::argopt_firefox_cmd,
+        %App::BrowserUtils::argopt_quiet,
+    },
+    features => {
+        dry_run => 1,
+    },
+};
+sub restart_firefox {
+    App::BrowserUtils::restart_browsers(@_, restart_firefox=>1);
+}
+
+$SPEC{start_firefox} = {
+    v => 1.1,
+    summary => "Start firefox if not already started",
+    args => {
+        %App::BrowserUtils::argopt_firefox_cmd,
+        %App::BrowserUtils::argopt_quiet,
+    },
+    features => {
+        dry_run => 1,
+    },
+};
+sub start_firefox {
+    App::BrowserUtils::start_browsers(@_, start_firefox=>1);
+}
+
 1;
 # ABSTRACT: Utilities related to Firefox
 
@@ -116,7 +158,7 @@ App::FirefoxUtils - Utilities related to Firefox
 
 =head1 VERSION
 
-This document describes version 0.012 of App::FirefoxUtils (from Perl distribution App-FirefoxUtils), released on 2020-05-24.
+This document describes version 0.013 of App::FirefoxUtils (from Perl distribution App-FirefoxUtils), released on 2020-06-13.
 
 =head1 SYNOPSIS
 
@@ -126,6 +168,8 @@ This distribution includes several utilities related to Firefox:
 
 =over
 
+=item * L<firefox-has-processes>
+
 =item * L<firefox-is-paused>
 
 =item * L<firefox-is-running>
@@ -140,6 +184,10 @@ This distribution includes several utilities related to Firefox:
 
 =item * L<ps-firefox>
 
+=item * L<restart-firefox>
+
+=item * L<start-firefox>
+
 =item * L<terminate-firefox>
 
 =item * L<unpause-firefox>
@@ -149,6 +197,42 @@ This distribution includes several utilities related to Firefox:
 =head1 FUNCTIONS
 
 
+=head2 firefox_has_processes
+
+Usage:
+
+ firefox_has_processes(%args) -> [status, msg, payload, meta]
+
+Check whether Firefox has processes.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<quiet> => I<true>
+
+=item * B<users> => I<array[unix::local_uid]>
+
+Kill browser processes that belong to certain user(s) only.
+
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (payload) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+
 =head2 firefox_is_paused
 
 Usage:
@@ -240,7 +324,7 @@ A modern browser now runs complex web pages and applications. Despite browser's
 power management feature, these pages/tabs on the browser often still eat
 considerable CPU cycles even though they only run in the background. Stopping
 (kill -STOP) the browser processes is a simple and effective way to stop CPU
-eating on Unix. It can be performed whenever you are not using your browsers for
+eating on Unix. It can be performed whenever you are not using your browser for
 a little while, e.g. when you are typing on an editor or watching a movie. When
 you want to use your browser again, simply unpause it.
 
@@ -304,6 +388,100 @@ Return value:  (any)
 
 
 
+=head2 restart_firefox
+
+Usage:
+
+ restart_firefox(%args) -> [status, msg, payload, meta]
+
+Restart firefox.
+
+This function is not exported.
+
+This function supports dry-run operation.
+
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<firefox_cmd> => I<array[str]|str> (default: "firefox")
+
+=item * B<quiet> => I<true>
+
+
+=back
+
+Special arguments:
+
+=over 4
+
+=item * B<-dry_run> => I<bool>
+
+Pass -dry_run=E<gt>1 to enable simulation mode.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (payload) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+
+=head2 start_firefox
+
+Usage:
+
+ start_firefox(%args) -> [status, msg, payload, meta]
+
+Start firefox if not already started.
+
+This function is not exported.
+
+This function supports dry-run operation.
+
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<firefox_cmd> => I<array[str]|str> (default: "firefox")
+
+=item * B<quiet> => I<true>
+
+
+=back
+
+Special arguments:
+
+=over 4
+
+=item * B<-dry_run> => I<bool>
+
+Pass -dry_run=E<gt>1 to enable simulation mode.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (payload) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+
 =head2 terminate_firefox
 
 Usage:
